-module(mulServerReg).
-export([start/0, mul_server/1, client/0]).

mul_server(Creator) ->

    % timer:sleep(1000),
    receive 
        % return messages 'tagged' with an id
        {Id, Pid, X, Y} ->
            Pid ! {Id, X*Y},
            mul_server(Creator);

        % stop message: only the creator can stop the server
        {Creator, stop} ->
            io:format("Server stopping ...~n", []);
           
        {Pid, stop} ->
            io:format("Process \"~w\" not allowed to stop the server ...~n", [Pid]),
            mul_server(Creator)
                                    
    end.

% facilities for outputting the result
out_result({_, _, Arg1, Arg2}, fail) ->
    io:format("Timed out waiting for the result of ~B * ~B~n", [Arg1,Arg2]);
out_result({_, _, Arg1, Arg2}, Result) ->
    io:format("Product of ~B * ~B is ~B~n", [Arg1, Arg2, Result]).
    

client() ->
    % first message
    % message id 
    Id1 = crypto:strong_rand_bytes(5),
    Msg1 = {Id1, self(), 2, 2},
    multiplier ! Msg1,

    receive 
        {Id1,P1} -> 
	    out_result(Msg1,P1)
    after 
     	10 ->
     	    out_result(Msg1,fail)
    end,
    
    multiplier ! {self(), stop },
    

    % second message (receive order could be swapped)
    Id2 = crypto:strong_rand_bytes(5),
    Msg2 = {Id2, self(), 4, 5},
    multiplier ! Msg2,
    
    receive 
        {Msg1,P2} -> 
	    out_result(Msg2,P2)
    after 
     	10 ->
     	    out_result(Msg2,fail)
    end.
    
    

start() ->
    % the server is registered
    Server = spawn(mulServerReg, mul_server, [self()]),
    register(multiplier, Server),

    % accessible to some client, without getting the pid
    spawn(mulServerReg, client, []),

    % wait a bit and stops the server
    timer:sleep(100),
    Server ! {self(),stop}.

